 /*
  *  T[Component] (see class name)
  *
  * Created by: Alexander Melnikov
  *  e-mail: alm@tolmachevo.ru
  *  Modified by:
  *  Last updated:
  *
  *
  * Copyright: 1998 ALM(COFF).
  *
  * Modifications Copyright:
  *
  * Permission granted to copy, modify, and otherwise use the modifications
  *  to this code in whatever manner you see fit, with no warranty expressed or
  *  implied.  Please retain this notice; this is the only restriction.If you
  *  make any changes to the original code and/or modifications put forth by
  *  ALM(COFF), plase credit yourself so that ALM(COFF) is not asked to
  *  maintain versions of this modified code that were not developed by
  *  ALM(COFF).
  *
  * ALM(COFF) GIVES NO WARRANTY, EXPRESSED OR IMPLIED, FOR THE SOFTWARE
  *  AND/OR DOCUMENTATION PROVIDED, INCLUDING, WITHOUT LIMITATION, WARRANTY
  *  OF MERCHANTABILITY AND WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE.
  *
 */

#include <vcl.h>
#include <FileCtrl.hpp>
#pragma hdrstop

#include "FSNotify.h"
#pragma package(smart_init)
// ---------------------------------------------------------------------------
/*
 * ValidCtrCheck is used to assure that the components created do not have
 * any pure virtual functions.
*/

static inline void
ValidCtrCheck(TFSNotify *)
{
  new             TFSNotify((TComponent *) NULL);
}

// ---------------------------------------------------------------------------
namespace       Fsnotify
{
  void __fastcall PACKAGE Register() {
    TComponentClass classes[1] =
    {__classid(TFSNotify)};
                    RegisterComponents("COFF", classes, 0);
  }
}
// ---------------------------------------------------------------------------
                Notify::
Notify(const String & Path, bool WSubTree, TNotifyFilters Filters, DWORD TimeOut)
{
  static DWORD    TNotifyFilter[] =
  {FILE_NOTIFY_CHANGE_FILE_NAME,
    FILE_NOTIFY_CHANGE_DIR_NAME,
    FILE_NOTIFY_CHANGE_ATTRIBUTES,
    FILE_NOTIFY_CHANGE_SIZE,
    FILE_NOTIFY_CHANGE_LAST_WRITE,
  FILE_NOTIFY_CHANGE_SECURITY};

  DWORD           dwFilters = 0;
  FTimeOut = TimeOut;

Fsnotify::TNotifyFilter Filter = cFileName;

  for (; Filter < cSecurity;
         Filter = Fsnotify::TNotifyFilter(Filter + 1))
         if (Filters.Contains(Filter))
             dwFilters = dwFilters | TNotifyFilter[Filter];

  FResult = true;

  FHandle = FindFirstChangeNotification(Path.c_str(), WSubTree, dwFilters);

  if (FHandle == INVALID_HANDLE_VALUE)
    FResult = false;

}
// ---------------------------------------------------------------------------
bool            Notify::
Next()
{
  if (!FResult)
    return false;

  DWORD           Status = WaitForSingleObject(FHandle, FTimeOut);
  switch (Status) {
  case WAIT_OBJECT_0:
  case WAIT_TIMEOUT:		// logic

    FResult = FindNextChangeNotification(FHandle);
    break;
  case WAIT_ABANDONED:
  default:
    break;
  }
  return FResult;
}
//---------------------------------------------------------------------------
__fastcall      TThNotifyWalker::
TThNotifyWalker():TThread(true)
{
  FPath = GetCurrentDir();
  FTimeOut = INFINITE;
  FSync = true;
  FFirst = false;
  FWSubTree = false;
  FFilters << cFileName;
}
// ---------------------------------------------------------------------------
void __fastcall TThNotifyWalker::
Execute(void)
{

  if (DirectoryExists(FPath)) {
    Notify         *cNF = new Notify(FPath, FWSubTree, FFilters, FTimeOut);

    if (FFirst)
      if (FSync)
	Synchronize(SendEvent);
      else
	SendEvent();

    while (cNF->Next() && !Terminated) {
      if (FSync)
	Synchronize(SendEvent);
      else
	SendEvent();
    }

    if (!Terminated) {
      FError = eContinue;
      FOptMsg = FPath;
      Synchronize(Error);
    }
  } else {
    FError = eDirExist;
    FOptMsg = FPath;
    Synchronize(Error);
  }

  FreeOnTerminate = true;
}
// ---------------------------------------------------------------------------
void __fastcall TThNotifyWalker::
Error()
{
  static char    *EMessage[] = {
    "Directory must be exist ",
    "Not a Continued notify  "};

  if (FOnError) {
    FOnError(this, FError,EMessage[FError] + FOptMsg);
  }
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void __fastcall TFSNotify::
SetFilters(TNotifyFilters AFilters)
{
  if(!FActive)
    FFilters = AFilters;
  else
  {
    Active = false;
    FFilters = AFilters;
    Active = true;
  }
}
//---------------------------------------------------------------------------
void __fastcall TFSNotify::
SetActive(bool AActive)
{
  if(FActive == AActive)
     return;

  if (!AActive) {
    if (FWlk != NULL) {
      FWlk->Terminate();
     // delete          FWlk;
    }
  } else {
    FWlk = new TThNotifyWalker();
    FWlk->Path = FPath;
    FWlk->TimeOut = FTimeOut;
    FWlk->Sync = true;
    FWlk->First = FFirst;
    FWlk->WSubTree = FSubTree;
    FWlk->OnError = FOnError;
    FWlk->OnChange = FOnEvent;
    FWlk->Filters = FFilters;
    FWlk->Resume();
  }
  FActive = AActive;
}
//---------------------------------------------------------------------------
void __fastcall TFSNotify::
SetPath(String APath)
{
  if(!FActive)
  FPath = APath;
  else
  {
    Active = false;
    FPath = APath;
    Active = true;
  }
}
//---------------------------------------------------------------------------
__fastcall TFSNotify::
TFSNotify(TComponent * Owner):TComponent(Owner)
{
  FActive = false;
  FTimeOut = INFINITE;
  FFilters << cFileName;
}
//---------------------------------------------------------------------------
__fastcall      TFSNotify::~
TFSNotify()
{
  if (FActive == true)
    Active = false;
}
//---------------------------------------------------------------------------
